PROGRAM_NAME='HDanywhere_mHub_Demo'
(***********************************************************)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 2016/01/05                      *)
(***********************************************************)
(* System Type : NetLinx                                   *)
(***********************************************************)
(* REV HISTORY:                                            *)
(***********************************************************)
(*
    $History: $
*)

DEFINE_DEVICE
 dvNetworking = 0:70:0
 dvTP = 10001:1:0;
 vdvHDanywhere = 33002:1:0; 

DEFINE_VARIABLE
 INTEGER g_inputToSet; //Bonded to DEVLEV:inputToSet
 volatile CHAR IPOrSerial = 1;
 volatile CHAR beepOnOff;
 volatile CHAR powerOnOff;
 volatile CHAR deviceAddress[] = '172.16.112.95'; //Base IP Address of our configured system
  
 volatile DEVCHAN buttons[] = { {dvTP, 1}, {dvTP, 2}, {dvTP, 3}, {dvTP, 4} } //The multi-switch buttons
 volatile DEVCHAN directApplyButtons[] = { {dvTP, 11}, {dvTP, 12}, {dvTP, 13}, {dvTP, 14} }; //The direct-switch buttons
 volatile DEVLEV inputToSet = { dvTP, 1 }; //Allows you to set which input we want output devices to display
 volatile DEVCHAN multiApply = { dvTP, 2000 }; //Triggers a multi-switch
 volatile DEVCHAN Power = {dvTP, 2001};
 volatile DEVCHAN Beep = {dvTP, 2002};
 
(* Adds a range of multi-switch numbers onto a string *)
DEFINE_CALL 'caAppendRange' ( CHAR switch_command[ ], INTEGER startRange, INTEGER endRange )
{
  if (startRange = endRange)
  {
    if (switch_command == "")
    {
      switch_command = "itoa(startRange)";
    }
    else
    {
      switch_command = "switch_command, ',', itoa(startRange)";
    }
  }
  else
  {
    if (switch_command == "")
    {
      switch_command = "itoa(startRange), '-', itoa(endRange)";
    }
    else
    {
      switch_command = "switch_command, ',', itoa(startRange), '-', itoa(endRange)";
    }      
  }
}

define_module 'HDanywhere_mHub_88'  HDanywhere_mHub( vdvHDanywhere, dvNetworking, deviceAddress, IPOrSerial );

DEFINE_START
{
  CREATE_LEVEL inputToSet, g_inputToSet;
	g_inputToSet = 1;
  SEND_LEVEL inputToSet, 1;
}

DEFINE_EVENT

(* Multi-apply buttons *)
(* Multi-Switch buttons *)
button_event[buttons]
{
  push:
  {
    INTEGER Index;
    Index = GET_LAST(buttons);

    if (0 == [buttons[Index]])
    {
     [buttons[Index]] = 1;
    }
    else
    {
     [buttons[Index]] = 0;
    }
  }
}
(* Apply the multi-switch *)
(* Sets multiple outputs to an input *)
button_event[multiApply]
{
  push:
  {
    INTEGER Len, i, startRange, endRange;
    CHAR switch_command[32000];
    Len = LENGTH_ARRAY(buttons);
    startRange = 0;
    switch_command = "";
    
    //Construct a switch string
    FOR (i = 1; i <= Len; i++)
    {
      if (0 != [buttons[i]])
      {
        [buttons[i]] = 0;
        
        if (startRange = 0)
        {
          startRange = i;
        }
  
        endRange = i;
      }
      else
      {
        if (startRange != 0)
        {
          CALL 'caAppendRange' (switch_command, startRange, endRange);
          startRange = 0;
        }
      }
    }
    //If a range was started but not finished by the time we reached the end of the array
    //You must add the last range onto the string
    if (startRange != 0)
    {
      CALL 'caAppendRange' (switch_command, startRange, endRange);
    }
    //Finally send the command into the Raritan module
    if (switch_command != "")
    {
      switch_command = "switch_command, '=', itoa(g_inputToSet), ';'";
      send_command vdvHDanywhere, switch_command;
    }
  }
}
(* Direct switch buttons *)
(* Directly sets a single output to an input *)
button_event[directApplyButtons]
{
  push:
  {
    INTEGER Index;
    Index = GET_LAST(directApplyButtons);
    send_command vdvHDanywhere, "itoa(Index), '=', itoa(g_inputToSet), ';'";
  }
}

(* The Slider for selecting which input *)
level_event[inputToSet]
{
  if (LEVEL.VALUE < 1)
  {
    send_level inputToSet, 1;
  }
  else if (LEVEL.VALUE > 4)
  {
    send_level inputToSet, 10;
  }
  else
  {
    send_command dvTP,"'^TXT-2000,0,', itoa(g_inputToSet)";
  }
}

button_event[Beep]
{
  push:
  {
    if (0 == beepOnOff)
    {
		  beepOnOff = 1;
			send_command dvTP,"'^TXT-2002,0,', 'Set Beep Off'";
     [vdvHDanywhere,2] = 1;
    }
    else
    {
		  beepOnOff = 0;
			send_command dvTP,"'^TXT-2002,0,', 'Set Beep On'";
     [vdvHDanywhere,2] = 0;
    }
  }
}

button_event[Power]
{
  push:
  {
	  send_string 0, "'Ici'";
    if (0 == powerOnOff)
    {
		  powerOnOff = 1;
			send_command dvTP,"'^TXT-2001,0,', 'Set Power Off'";
     [vdvHDanywhere,1] = 1;
    }
    else
    {
		  powerOnOff = 0;
			send_command dvTP,"'^TXT-2001,0,', 'Set Power On'";
     [vdvHDanywhere,1] = 0;
    }
  }
}

data_event[vdvHDanywhere]
{
  string:
	{
	  Char Substr[64];
		Substr = REMOVE_STRING(data.text, '=', 1);
		Substr = LEFT_STRING(Substr, LENGTH_STRING(Substr)-1);
		data.text = LEFT_STRING(data.text, LENGTH_STRING(data.text)-1);
		send_command dvTP,"'^TXT-', Substr,',0,00', data.text";
  }
}

(* Status event of a touch panel connecting *)
data_event[dvTP]
{
  online:
  {
    //Because there's an AMX "Feature" that resets all levels to 0 when a touch panel disconnects
    //And it doens't fire the events... Ergo avoiding our validation!
    if (0 == g_inputToSet)
    {
      g_inputToSet = 1;
    }
		if (0 == powerOnOff)
    {
			send_command dvTP,"'^TXT-2001,0,', 'Set Power Off'";
    }
    else
    {
			send_command dvTP,"'^TXT-2001,0,', 'Set Power On'";
    }
		if (0 == beepOnOff)
    {
			send_command dvTP,"'^TXT-2002,0,', 'Set Beep Off'";
    }
    else
    {
			send_command dvTP,"'^TXT-2002,0,', 'Set Beep On'";
    }
    send_command dvTP,"'^TXT-2000,0,', itoa(g_inputToSet)";
  }
}
